@@ -1,8 +1,7 @@
 package Catalyst::DispatchType;
 
-use Moose;
-with 'MooseX::Emulate::Class::Accessor::Fast';
-no Moose;
+use strict;
+use base 'Class::Accessor::Fast';
 
 =head1 NAME
 
@@ -14,8 +13,6 @@ See L<Catalyst>.
 
 =head1 DESCRIPTION
 
-This is an abstract base class for Dispatch Types. 
-
 =head1 METHODS
 
 =head2 $self->list($c)
@@ -59,18 +56,10 @@ arrayref, or undef if unable to do so.
 
 sub uri_for_action { }
 
-=head2 $self->expand_action
+=head1 AUTHOR
 
-Default fallback, returns nothing. See L<Catalyst::Dispatcher> for more info
-about expand_action.
-
-=cut
-
-sub expand_action { }
-
-=head1 AUTHORS
-
-Catalyst Contributors, see Catalyst.pm
+Matt S Trout
+Sebastian Riedel, C<sri@cpan.org>
 
 =head1 COPYRIGHT
 
@@ -79,6 +68,4 @@ the same terms as Perl itself.
 
 =cut
 
-__PACKAGE__->meta->make_immutable;
-
 1;
